@@ -61,6 +61,11 @@ class Config:
         default_headers.update(kwargs.pop('headers', {}))
         return requests.post(self.url(path), headers=default_headers, **kwargs)
 
+    def put(self, path, **kwargs):
+        default_headers = self.default_headers
+        default_headers.update(kwargs.pop('headers', {}))
+        return requests.put(self.url(path), headers=default_headers, **kwargs)
+
 
     def get(self, path, **kwargs):
         default_headers = self.default_headers
@@ -145,7 +150,7 @@ def submit(ctx, file, project, title, test):
 
 
 @cli.command()
-@click.argument('package', required=False)
+@click.argument('package', required=False, metavar="PACKAGE")
 @click.option('--name', '-n', help="Optional name for your zip file")
 @click.pass_context
 def release(ctx, name=None, package=None):
@@ -181,8 +186,9 @@ def release(ctx, name=None, package=None):
 
 
 @cli.command("upload")
-@click.argument('archive', required=True, type=click.Path(exists=True))
-@click.argument('package', required=True)
+@click.argument('archive', required=True, type=click.Path(exists=True),
+                 metavar="ARCHIVE")
+@click.argument('package', required=True, metavar="PACKAGE")
 @click.pass_context
 def upl(ctx, archive, package):
     """Upload an existing archive to an existing package"""
@@ -254,10 +260,10 @@ def upl(ctx, archive, package):
         bar.update(30)
         if errors:
             click.echo("\nPackage upload to %s unsuccessful. "
-                       "The following error was "
-                       "returned: %s" %
+                       "The following error(s) was "
+                       "returned: \n%s" %
                        (get_package_name(package_id),
-                        (',').join(e.get('message', '[Unknown]') for
+                        ('\n').join(e.get('message', '[Unknown]') for
                                    e in errors)))
         else:
             click.echo("\nPackage uploaded successfully! \n"
@@ -286,11 +292,14 @@ def packages(ctx, i):
         for category, packages in package_names.items():
             if category == "yours":
                 click.echo('\n{:^80}'.format("YOUR PACKAGES:"))
-            else:
-                click.echo('\n{:^80}'.format("OTHER PACKAGES IN YOUR ORG:"))
-            click.echo('{:^40}'.format("PACKAGE NAME") + "|" +
+                click.echo('{:^40}'.format("PACKAGE NAME") + "|" +
                        '{:^40}'.format("PACKAGE ID"))
-            click.echo('{:-^80}'.format(''))
+                click.echo('{:-^80}'.format(''))
+            elif category == "theirs" and packages.values():
+                click.echo('\n{:^80}'.format("OTHER PACKAGES IN YOUR ORG:"))
+                click.echo('{:^40}'.format("PACKAGE NAME") + "|" +
+                           '{:^40}'.format("PACKAGE ID"))
+                click.echo('{:-^80}'.format(''))
             for name, id in packages.items():
                 click.echo('{:<40}'.format(name) + "|" +
                            '{:^40}'.format(id))
@@ -320,6 +329,21 @@ def new_package(ctx, description, name):
     else:
         click.echo("There was an error creating this package.")
 
+@cli.command("delete-package")
+@click.argument('name')
+@click.option('--force', '-f', help="force delete a package without being prompted if you're sure", is_flag=True)
+@click.pass_context
+def delete_package(ctx, name, force):
+    '''Create a new empty protocol package'''
+    id = get_package_id(name)
+    if id:
+        if not force:
+            click.confirm("Are you sure you want to permanently delete the package"
+                          " '%s'?  All releases within will be lost." %
+                          get_package_name(id), default=False, abort=True)
+            click.confirm("Are you really really sure?", default=True)
+        del_pack = ctx.obj.delete('/packages/%s' % id)
+        click.echo("Package deleted.")
 
 @cli.command()
 @click.pass_context
@@ -328,27 +352,40 @@ def projects(ctx, i):
     '''List the projects in your organization'''
     response = ctx.obj.get('')
     proj_names = {}
+    proj_cats = {"reg": {}, "pilot": {}}
     if response.status_code == 200:
         for proj in response.json()['projects']:
             proj_names[proj['name']] =  proj['id']
+            if proj.get("is_developer"):
+                proj_cats["pilot"][proj['name']] =  proj['id']
+            else:
+                proj_cats["reg"][proj['name']] =  proj['id']
         if i:
             return {k.lower(): v for k,v in proj_names.items()}
         else:
-            click.echo('{:^35}'.format("PROJECT NAME") + "|" +
-                       '{:^35}'.format("PROJECT ID"))
-            click.echo('{:-^70}'.format(''))
-            for name, i in proj_names.items():
-                click.echo('{:<35}'.format(name) + "|" +
-                           '{:^35}'.format(i))
-                click.echo('{:-^70}'.format(''))
+            for cat, packages in proj_cats.items():
+                if cat == "reg":
+                    click.echo('\n{:^80}'.format("PROJECTS:"))
+                    click.echo('{:^40}'.format("PROJECT NAME") + "|" +
+                               '{:^40}'.format("PROJECT ID"))
+                    click.echo('{:-^80}'.format(''))
+                elif cat == "pilot" and packages.values():
+                    click.echo('\n{:^80}'.format("PILOT PROJECTS:"))
+                    click.echo('{:^40}'.format("PROJECT NAME") + "|" +
+                               '{:^40}'.format("PROJECT ID"))
+                    click.echo('{:-^80}'.format(''))
+                for name, i in packages.items():
+                    click.echo('{:<40}'.format(name) + "|" +
+                               '{:^40}'.format(i))
+                    click.echo('{:-^80}'.format(''))
     else:
         click.echo("There was an error listing the projects in your "
                    "organization.  Make sure your login details are correct.")
 
 
 @cli.command("new-project")
-@click.argument('name')
-@click.option('dev', '-d', help="Create a pilot project", is_flag=True)
+@click.argument('name', metavar="PROJECT_NAME")
+@click.option('--dev', '-d', '-pilot', help="Create a pilot project", is_flag=True)
 @click.pass_context
 def new_project(ctx, name, dev):
     '''Create a new empty project'''
@@ -371,8 +408,8 @@ def new_project(ctx, name, dev):
 
 
 @cli.command("delete-project")
-@click.argument('name')
-@click.option('force', '-f', help="force delete a project without being prompted if you're sure", is_flag=True)
+@click.argument('name', metavar="PROJECT_NAME")
+@click.option('--force', '-f', help="force delete a project without being prompted if you're sure", is_flag=True)
 @click.pass_context
 def delete_project(ctx, name, force):
     '''Delete an existing package'''
@@ -383,9 +420,14 @@ def delete_project(ctx, name, force):
                           default=False,
                           abort=True)
         dele = ctx.obj.delete('%s' % id, data=json.dumps({"id": id}))
-
-        click.echo("Project deleted.")
-
+        if dele.status_code == 200:
+            click.echo("Project deleted.")
+        else:
+            click.confirm("You cannot delete a project that contains runs.  Archive it instead?",
+                          default=False, abort=True)
+            arch = ctx.obj.put('%s' % id, data=json.dumps({"project": {"archived": True}}))
+            if arch.status_code == 200:
+                click.echo("Project archived.")
 
 
 @cli.command()
@@ -464,7 +506,7 @@ def price(response):
         click.echo("WARNING (%s): %s" % (context, message))
 
 @cli.command()
-@click.argument('protocol_name')
+@click.argument('protocol_name', metavar="PROTOCOL_NAME")
 def preview(protocol_name):
     '''Preview the Autoprotocol output of a script'''
     with click.open_file('manifest.json', 'r') as f:
@@ -515,7 +557,7 @@ def summarize(ctx, file):
 
 
 @cli.command()
-@click.argument('protocol_name')
+@click.argument('protocol_name', metavar="PROTOCOL_NAME")
 @click.argument('args', nargs=-1)
 def run(protocol_name, args):
     '''Run a protocol by passing it a config file (without submitting or analyzing)'''
@@ -634,7 +676,7 @@ def get_package_id(ctx, name):
     package_id = package_names.get(name)
     if not package_id:
         package_id = name if name in package_names.values() else None
-    if not package_id and __name__ == "__main__":
+    if not package_id:
         click.echo("The package %s does not exist in your organization." % name)
         return
     return package_id
@@ -646,7 +688,7 @@ def get_package_name(ctx, id):
     package_name = package_names.get(id)
     if not package_name:
         package_name = id if id in package_names.values() else None
-    if not package_name and __name__ == "__main__":
+    if not package_name:
         click.echo("The id %s does not match any package in your organization."
                    % id)
         return
